;------------------------------------------------------------
;------------------------------------------------------------
;------------------------------------------------------------
;+
; NAME:union
;
; PURPOSE:calcule l''union de 2 matrices D'ENTIERS POSITIFS
;
; CATEGORY:calcule sur les matrices
;
; CALLING SEQUENCE:res=union(a,b)
;
; INPUTS:a et b:arrays of positive integers, which need
;               not be sorted. Duplicate elements are ignored, as they have no
;               effect on the result
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:tableau
;
; COMMON BLOCKS:
;
; SIDE EFFECTS:
;
; The empty set is denoted by an array with the first element equal to
; -1.
;
; RESTRICTIONS:
;
; These functions will not be efficient on sparse sets with wide
; ranges, as they trade memory for efficiency. The HISTOGRAM function
; is used, which creates arrays of size equal to the range of the
; resulting set.
;
; EXAMPLE:
;
;   a = [2,4,6,8]
;   b = [6,1,3,2]
;   union(a,b) = [ 1, 2, 3, 4, 6, 8]  ; Elements in either set
;
; MODIFICATION HISTORY: 
;
; http://www.dfanning.com/tips/set_operations.html
;-
;------------------------------------------------------------
;------------------------------------------------------------
;------------------------------------------------------------
FUNCTION union, a, b
;
  compile_opt idl2, strictarrsubs
;
IF a[0] LT 0 THEN RETURN, b    ;A union NULL = a
IF b[0] LT 0 THEN RETURN, a    ;B union NULL = b
RETURN, Where(Histogram([a,b], OMin = omin)) + omin ; Return combined set
END
